const axios = require('axios')
const flatten = require('flat')
const CustomError = require('../errors')
const CRM_Contact = require('../models/CRM_Contact')
const CRM = require('../models/CRM')

const { URL } = require('url')
const { getAccessToken } = require('./oauth2')
const findProperty = require('./findProperty')
const requestConfigHeader = require('./requestConfigHeader')
const sendMessageSocket = require('./sendMessageSocket')
const CRM_Ticket = require('../models/CRM_Ticket')

const getHubspotPipelines = require('./getHubspotPipelines')
const { pipeline } = require('stream')


async function lookupCrmTicket(rest, authentication, crmPhone, companyId, test = {}, ticketId) {

    let { request, body, response } = findProperty(rest, 'lookupTicket')

    let { requestContentType, requestEncoding, requestType, responseType, url } = request


    const { type, userName, passWord, token, crmClientId } = authentication

    const config = await requestConfigHeader(url, crmPhone, requestType, requestContentType, type, userName, passWord, token, crmClientId, '', ticketId, companyId)

    if (test?.testing) {
        let msg = `Tentanto checar o status do ticket para o numer ${crmPhone} no crm`
        sendMessageSocket({ companyId, status: 'processing', data: { request: config, msg } })
    }

    let resp

    try {
        resp = await axios(config)

        console.log(`====> CompanyId: ${companyId} Request config para consultar status do ticket\n: `, JSON.stringify(config, null, 6))
        console.log(`====> CompanyId: ${companyId} Response consultar status do ticket\n: `, JSON.stringify(resp.data, null, 6))

    } catch (error) {

        if (error.response) {
            console.error('==================> lookupCrmTicket Erro na resposta da API:', {
                status: error.response.status,
                data: error.response.data,
            })
        }
        else if (error.request) {
            console.error('==================> lookupCrmTicket Nenhuma resposta recebida da API:', error?.request)
        }
        else {
            console.error('==================> lookupCrmTicket Erro ao configurar a request:', error?.message)
        }

        if (error?.response?.status == 404)
            return { error: 404 }
    }

    let { data } = resp

    data = flatten(data)

    let auxTicketStatus
    let auxTicketId

    for (const prop in data) {

        const _prop = prop.replace(/^\d+\./, '').replace(/(?:^|\.)\d+\b/g, '')

        if (_prop == response?.status?.trim()) {
            auxTicketStatus = data[prop].replace('+', '')
        }

        if (_prop == response?.id?.trim()) {
            auxTicketId = data[prop]
        }

        if (auxTicketStatus && auxTicketId) break

    }

    if (!auxTicketStatus && !auxTicketId) {
        for (const prop in data) {

            let _prop = prop.replace(/\.(\d+)(\.|$)/g, '[$1]$2')

            if (_prop == response?.status?.trim()) {
                auxTicketStatus = data[prop].replace('+', '')
            }

            if (_prop == response?.id?.trim()) {
                auxTicketId = data[prop]
            }

            if (auxTicketStatus && auxTicketId) break

        }
    }

    console.log('auxTicketStatus: ', auxTicketStatus, ' | auxTicketId: ', auxTicketId)

    if (url.includes('api.hubapi.com')) {
        const pipelines = await getHubspotPipelines(token)

        let pipeline

        for (let pip of pipelines.results) {
            pipeline = pip.stages.find(({ id }) => id == auxTicketStatus)

            if (pipeline) {
                const { ticketState } = pipeline.metadata
                const { label } = pipeline

                console.log(pipeline)

                pipeline = { ticketState, label }
                break
            }
        }

        console.log('=========> Hubspot pipeline: ', pipeline)

        const {ticketState, label} = pipeline

        // o label esta sendo usado aqui poque no hubspot padrao que estamos usando mesmo que o ticket esteja
        // fechado ele continua com ticketState OPEN ao inves de CLOSED
        if (ticketState == 'OPEN' && label != "Closed"){
            console.log ('**** TICKET ESTA ABERTO ****')
            return { auxTicketId, auxTicketStatus: 1 } 
        }

    }

    return { auxTicketId, auxTicketStatus }

}

module.exports = lookupCrmTicket


